from django.views import generic

from .base import TemplateNameResolverMixin


__all__ = (
    'CreateView',
    'CreateViewMixin',
    'DetailView',
    'DetailViewMixin',
    'UpdateView',
    'UpdateViewMixin',
    'DeleteView',
    'DeleteViewMixin',
)


class CreateViewMixin(TemplateNameResolverMixin):
    template_suffixes = ('create', 'editing', 'form')   


class CreateView(CreateViewMixin, generic.CreateView):    
    pass


class DetailViewMixin(TemplateNameResolverMixin):
    template_suffixes = ('detail',) 


class DetailView(DetailViewMixin, generic.DetailView):
    pass


class UpdateViewMixin(TemplateNameResolverMixin):
    template_suffixes = ('update', 'editing', 'form')   


class UpdateView(UpdateViewMixin, generic.UpdateView):    
    pass


class DeleteViewMixin(TemplateNameResolverMixin):
    template_suffixes = ('confirm-delete',) 


class DeleteView(DeleteViewMixin, generic.DeleteView):
    pass



from django.views import generic

from .base import TemplateNameResolverMixin


__all__ = (
    'ListView',
)


class ListViewMixin(TemplateNameResolverMixin):    
    template_suffixes = ('list',)


class ListView(ListViewMixin, generic.ListView):
    pass



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-canclon',
    version='0.1.3',
    packages=['canclon'],
    include_package_data=True,
    license='MIT',
    description='Template name resolution for Django class-based views',
    long_description=README,
    url='https://github.com/jleeothon/canclon',
    author='Johnny Lee',
    author_email='jleeothon@outlook.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=['django'],
    tests_require=['django'],
)


__all__ = (
    'TemplateNameResolverMixin',
)


class TemplateNameResolverMixin(object):
    """
    Overrides get template name.
    """
    
    template_suffixes = ('',)
    template_suffix_separators = ('-',)
    template_extensions = ('html',)

    def __template_name_parts(self):
        for suffix in self.template_suffixes:
            for separator in self.template_suffix_separators:
                for extension in self.template_extensions:
                    yield suffix, separator, extension

    def __format_template_names(self, suffix, separator, extension):
        args = (
            self.model._meta.app_label,
            self.model._meta.model_name,
            (separator + suffix if suffix else ""),
            extension,
        )
        name = '%s/%s%s.%s' % args
        return name

    def __template_names(self):
        """
        Generator function that resolves template names.
        """
        for suffix, separator, extension in self.__template_name_parts():
            yield self.__format_template_names(suffix, separator, extension)

    def get_template_names(self):
        if self.template_name is not None:
            return [self.template_name]
        template_names = list(self.__template_names())
        return template_names
